precision highp float;

attribute vec4 position;
attribute vec4 inputTextureCoordinate;
attribute vec4 inputTextureCoordinate2;
varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;

uniform mat4 uMVPMatrix;
uniform mat4 uTexMatrix;
uniform mat4 uTexMatrix2;

uniform vec2 iResolution;
uniform vec2 iResolution2;

uniform float randomSeed;
uniform int frameIndex;

const vec4 PositionRect = vec4(-1., -1., 2., 2.);
const vec4 UvRect = vec4(0., 0., 1., 1.);

/**
 *  Rect(x, y, width, height) to point rect(bottomLeft, topRight).
 */
vec4 rectToPRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.x + rect.z, rect.y + rect.w);
}

/**
 *  Point rect(bottomLeft, topRight) to rect(x, y, width, height).
 */
vec4 pRectToRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.z - rect.x, rect.w - rect.y);
}

vec2 transformPointByRect(vec4 srcRect, vec4 destRect, vec2 point) {
    return vec2(((point.x - srcRect.x) / srcRect.z) * destRect.z + destRect.x,
    ((point.y - srcRect.y) / srcRect.w) * destRect.w + destRect.y);
}

vec4 transformRectByRect(vec4 srcRect, vec4 destRect, vec4 rect) {
    vec4 pRect = rectToPRect(rect);
    return pRectToRect(vec4(transformPointByRect(srcRect, destRect, pRect.xy),
    transformPointByRect(srcRect, destRect, pRect.zw)));
}

vec2 transformPosToUvByRect(vec4 canvasRect, vec4 elementRect, vec2 pos) {
    return transformPointByRect(elementRect, UvRect, transformPointByRect(PositionRect, canvasRect, pos));
}

vec4 putRectToRectUsingNone(vec4 rect, vec2 size) {
    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniform(vec4 rect, vec2 size) {
    float scale = min(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniformToFill(vec4 rect, vec2 size) {
    float scale = max(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToPosition(vec2 size, vec2 pos) {
    return vec4(pos.x - size.x / 2.0, pos.y - size.y / 2.0, size);
}

vec4 centerRect(vec2 size) {
    return putRectToPosition(size, vec2(0.0));
}

vec4 resizeRect(vec4 rect, float scale) {
    vec2 newSize = rect.zw * scale;
    return vec4(rect.x + (rect.z - newSize.x) / 2.0,
    rect.y + (rect.w - newSize.y) / 2.0, newSize.x, newSize.y);
}

highp float rand1(highp float r) {
    return fract(sin(dot(vec2(r, r), vec2(12.9898, 78.233))) * 43758.5453);
}

highp float rand2(highp float r) {
    return fract(sin(dot(vec2(r, r), vec2(1.988, 78.233))) * 43758.5453);
}

void main() {
    vec4 rect = centerRect(iResolution);
    vec2 normalizedNoiseSize = iResolution2 / iResolution2.x * (min(iResolution.x, iResolution.y) / 6.);
    vec4 noiseRect = putRectToRectUsingNone(rect, normalizedNoiseSize);

    textureCoordinate = (uTexMatrix * inputTextureCoordinate).xy;
    vec2 noisePos = transformPointByRect(vec4(0., 0., 1., 1.), rect, textureCoordinate);
    textureCoordinate2 = transformPointByRect(noiseRect, vec4(0., 0., 1., 1.), noisePos);

    textureCoordinate2 += vec2(rand1(randomSeed + float(frameIndex) / 100.0), rand2(randomSeed + float(frameIndex) / 100.0));
    gl_Position = uMVPMatrix * position;
}