precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform vec4 resolution;
uniform float intensity;

vec4 getPin0(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}

vec4 getPin0Clamp(vec2 uv) {
    return getPin0(clamp(uv, 0.001, 0.999));
}

float SCurve(float x) {
    x = x * 2.0 - 1.0;
    return -x * abs(x) * 0.5 + x + 0.5;
}

vec4 BlurV(vec2 size, vec2 uv, float radius) {
    lowp vec4 A = vec4(0.0);
    vec4 C = vec4(0.0);

    float heightStep = 1.0 / size.y;

    float divisor = 0.0;
    float weight = 0.0;

    float radiusPixels = radius * min(size.x, size.y);
    float radiusMultiplier = 1.0 / radiusPixels;

    for (float y = -floor(radiusPixels); y <= floor(radiusPixels); y++) {
        A = getPin0Clamp(uv + vec2(0.0, y * heightStep));
        weight = SCurve(1.0 - (abs(y) * radiusMultiplier));
        C += A * weight;
        divisor += weight;
    }

    return C / divisor;
}

void main() {
    gl_FragColor = BlurV(resolution.xy, textureCoordinate, intensity);
}
