precision highp float;
#define PI 3.1415926535897932

uniform vec4 resolution;
uniform vec2 texture0Size;
uniform vec2 texture1Size;
uniform float angle;
varying vec2 imageRectUv;
varying vec2 imageMaskUv;

attribute vec4 position;
attribute vec4 inputTextureCoordinate;
attribute vec4 inputTextureCoordinate2;
varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;
uniform mat4 uMVPMatrix;
uniform mat4 uTexMatrix;
uniform mat4 uTexMatrix2;

//#include "util_rect.glsl"
vec2 fromPolar(vec2 rt) {
    return vec2(rt.x * cos(rt.y), rt.x * sin(rt.y));
}

vec2 toPolar(vec2 v) {
    return vec2(length(v), atan(v.y, v.x));
}

vec2 fromPolar(vec2 rt, vec2 orign) {
    return fromPolar(rt) + orign;
}

vec2 toPolar(vec2 v, vec2 orign) {
    return toPolar(v - orign);
}

vec2 rotatePoint(vec2 v, float angle) {
    return fromPolar(toPolar(v) + vec2(0, angle));
}

vec2 rotatePoint(vec2 v, float angle, vec2 orign) {
    return fromPolar(toPolar(v, orign) + vec2(0, angle), orign);
}

const vec4 PositionRect = vec4(-1., -1., 2., 2.);
const vec4 UvRect = vec4(0., 0., 1., 1.);

/**
 *  Rect(x, y, width, height) to point rect(bottomLeft, topRight).
 */
vec4 rectToPRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.x + rect.z, rect.y + rect.w);
}

/**
 *  Point rect(bottomLeft, topRight) to rect(x, y, width, height).
 */
vec4 pRectToRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.z - rect.x, rect.w - rect.y);
}

vec2 transformPointByRect(vec4 srcRect, vec4 destRect, vec2 point) {
    return vec2(((point.x - srcRect.x) / srcRect.z) * destRect.z + destRect.x,
    ((point.y - srcRect.y) / srcRect.w) * destRect.w + destRect.y);
}

vec4 transformRectByRect(vec4 srcRect, vec4 destRect, vec4 rect) {
    vec4 pRect = rectToPRect(rect);
    return pRectToRect(vec4(transformPointByRect(srcRect, destRect, pRect.xy),
    transformPointByRect(srcRect, destRect, pRect.zw)));
}

vec2 transformPosToUvByRect(vec4 canvasRect, vec4 elementRect, vec2 pos) {
    return transformPointByRect(elementRect, UvRect, transformPointByRect(PositionRect, canvasRect, pos));
}

vec4 putRectToRectUsingNone(vec4 rect, vec2 size) {
    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniform(vec4 rect, vec2 size) {
    float scale = min(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniformToFill(vec4 rect, vec2 size) {
    float scale = max(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToPosition(vec2 size, vec2 pos) {
    return vec4(pos.x - size.x / 2.0, pos.y - size.y / 2.0, size);
}

vec4 centerRect(vec2 size) {
    return putRectToPosition(size, vec2(0.0));
}

vec4 resizeRect(vec4 rect, float scale) {
    vec2 newSize = rect.zw * scale;
    return vec4(rect.x + (rect.z - newSize.x) / 2.0,
    rect.y + (rect.w - newSize.y) / 2.0, newSize.x, newSize.y);
}

void main() {
    vec4 canvasRect = centerRect(resolution.xy);
    vec2 imageMaskSize;
    vec4 imageMaskRect;
    vec4 imageRect;
    highp float rotate = (2.0 * PI) * (angle / 360.0);
    if (resolution.x <= resolution.y) {
        imageMaskSize = texture1Size / max(texture1Size.x / resolution.x, texture1Size.y / resolution.y) * 0.85;

        imageMaskRect = putRectToRectUsingNone(canvasRect, imageMaskSize);
        imageRect = putRectToRectUsingUniformToFill(imageMaskRect, texture0Size);
    } else {
        imageMaskSize = texture1Size.yx / max(texture1Size.y / resolution.x, texture1Size.x / resolution.y) * 0.85;

        imageMaskRect = putRectToRectUsingNone(canvasRect, imageMaskSize);
        imageRect = putRectToRectUsingUniformToFill(imageMaskRect, texture0Size);
    }
    imageMaskUv = transformPointByRect(vec4(-1., -1., 2., 2.), canvasRect, position.xy);
    imageMaskUv = rotatePoint(imageMaskUv, rotate);
    imageMaskUv = transformPointByRect(imageMaskRect, vec4(0., 0., 1., 1.),imageMaskUv);

    imageRectUv = transformPointByRect(vec4(-1., -1., 2., 2.), canvasRect, position.xy);
    imageRectUv = rotatePoint(imageRectUv, rotate);
    imageRectUv = transformPointByRect(imageRect, vec4(0., 0., 1., 1.),imageRectUv);

    gl_Position = uMVPMatrix * position;
    textureCoordinate = (uTexMatrix * inputTextureCoordinate).xy;
    textureCoordinate2 = (uTexMatrix2 * inputTextureCoordinate2).xy;
}