precision highp float;

#define __CONTEXT__

#define PI 3.141596
#define GOLDEN_ANGLE 2.39996323
#define NUMBER 50.0
#define ITERATIONS (GOLDEN_ANGLE * NUMBER)

varying vec2 textureCoordinate;
//图片纹理
uniform sampler2D inputImageTexture;
uniform vec2 iResolution;
uniform mat4 pin0Transform;
uniform highp float randomSeed;


//-------------------------------------------------------------------------------------------
// This creates the 2D offset for the next point.
// (r-1.0) is the equivalent to sqrt(0, 1, 2, 3...)
vec2 Sample(in float theta, inout float r) {
    r += 1.0 / r;
    return (r - 1.0) * vec2(cos(theta), sin(theta));
}

vec4 getPin0(vec2 uv, float bias) {
    return texture2D(inputImageTexture, vec4(clamp(uv, 0.001, 0.999), 1., 1.).xy);
}

//-------------------------------------------------------------------------------------------
vec4 Bokeh(vec2 uv, float radius, float amount) {
    highp vec4 acc = vec4(0.0);
    vec4 div = vec4(0.0);
    vec2 pixel = vec2(iResolution.y / iResolution.x, 1.0) * radius * .006;
    float r = 1.0;
    for (float j = 0.0; j < ITERATIONS; j += GOLDEN_ANGLE) {
        vec4 col = getPin0(uv + pixel * Sample(j, r), radius * 1.5);
        vec4 bokeh = vec4(5.0) + pow(col, vec4(9.0)) * amount;
        acc += col * bokeh;
        div += bokeh;
    }
    return acc / div;
}

void main() {
    float strength = mix(0.2, 0.2, step(0.9, randomSeed));
    float r = 0.7 * strength;
    float dis = distance(vec2(0.8), vec2(0.5));
    gl_FragColor = Bokeh(textureCoordinate, r * dis * 0.5 /*中间清晰的大小，越小越清晰*/, NUMBER);
}