attribute vec4 position;
attribute vec4 inputTextureCoordinate;
attribute vec4 inputTextureCoordinate2;
varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;
uniform mat4 uMVPMatrix;
uniform mat4 uTexMatrix;
uniform mat4 uTexMatrix2;

uniform vec2 texture0Size;
uniform vec2 texture1Size;
varying vec2 noiseUv;

const vec4 PositionRect = vec4(-1., -1., 2., 2.);
const vec4 UvRect = vec4(0., 0., 1., 1.);

float randomSeed = 1.0;
uniform int frameIndex;

/**
 *  Rect(x, y, width, height) to point rect(bottomLeft, topRight).
 */
vec4 rectToPRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.x + rect.z, rect.y + rect.w);
}

/**
 *  Point rect(bottomLeft, topRight) to rect(x, y, width, height).
 */
vec4 pRectToRect(vec4 rect) {
    return vec4(rect.x, rect.y, rect.z - rect.x, rect.w - rect.y);
}

vec2 transformPointByRect(vec4 srcRect, vec4 destRect, vec2 point) {
    return vec2(((point.x - srcRect.x) / srcRect.z) * destRect.z + destRect.x,
    ((point.y - srcRect.y) / srcRect.w) * destRect.w + destRect.y);
}

vec4 transformRectByRect(vec4 srcRect, vec4 destRect, vec4 rect) {
    vec4 pRect = rectToPRect(rect);
    return pRectToRect(vec4(transformPointByRect(srcRect, destRect, pRect.xy),
    transformPointByRect(srcRect, destRect, pRect.zw)));
}

vec2 transformPosToUvByRect(vec4 canvasRect, vec4 elementRect, vec2 pos) {
    return transformPointByRect(elementRect, UvRect, transformPointByRect(PositionRect, canvasRect, pos));
}

vec4 putRectToRectUsingNone(vec4 rect, vec2 size) {
    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniform(vec4 rect, vec2 size) {
    float scale = min(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToRectUsingUniformToFill(vec4 rect, vec2 size) {
    float scale = max(rect.z / size.x, rect.w / size.y);
    size = scale * size;

    return vec4(rect.x + (rect.z - size.x) / 2.0,
    rect.y + (rect.w - size.y) / 2.0, size);
}

vec4 putRectToPosition(vec2 size, vec2 pos) {
    return vec4(pos.x - size.x / 2.0, pos.y - size.y / 2.0, size);
}

vec4 centerRect(vec2 size) {
    return putRectToPosition(size, vec2(0.0));
}

vec4 resizeRect(vec4 rect, float scale) {
    vec2 newSize = rect.zw * scale;
    return vec4(rect.x + (rect.z - newSize.x) / 2.0,
    rect.y + (rect.w - newSize.y) / 2.0, newSize.x, newSize.y);
}

float rand(vec2 xy, float seed) {
    vec3 p3 = vec3(xy, seed);
    p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.zyx + 31.32);
    return fract((p3.x + p3.y) * p3.z);
}

float rand(vec2 xy) {
    vec3 p3  = fract(vec3(xy.xyx) * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

float rand(float seed) {
    seed = fract(seed * .1031);
    seed *= seed + 33.33;
    seed *= seed + seed;
    return fract(seed);
}

float nextRand(inout float seed) {
    seed = rand(vec2(0.64825 - rand(seed + 3.456), 0.83658 + rand(seed * 0.34)), 1. - seed);
    return seed;
}

void main()
{
    textureCoordinate = (uTexMatrix * inputTextureCoordinate).xy;
    textureCoordinate2 = (uTexMatrix2 * inputTextureCoordinate2).xy;

    vec4 rect = centerRect(texture0Size);
    vec2 normalizedPin2Size = texture1Size / texture1Size.x * (min(texture0Size.x, texture0Size.y) / 3.);
    vec4 noiseRect = putRectToRectUsingNone(rect, normalizedPin2Size);
    vec2 noisePos = transformPointByRect(vec4(0., 0., 1., 1.), rect, textureCoordinate);
    noiseUv = transformPointByRect(noiseRect, vec4(0., 0., 1., 1.), noisePos);
    noiseUv += vec2(rand(randomSeed + float(frameIndex)));
    gl_Position = uMVPMatrix * position;

}