precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float hardlightIntensity;
uniform float screenIntensity;
uniform float mixIntensity;

float hardLight(float base, float blend) {
    return blend<0.5?(2.0*blend*base):(1.0-2.0*(1.0-blend)*(1.0-base));
}

vec3 hardLight(vec3 base, vec3 blend) {
    return vec3(hardLight(base.r, blend.r), hardLight(base.g, blend.g), hardLight(base.b, blend.b));
}

vec3 hardLight(vec3 base, vec3 blend, float opacity) {
    return (hardLight(base, blend) * opacity + base * (1.0 - opacity));
}

vec4 hardLight(vec4 base, vec4 blend) {
    return vec4(
    hardLight(base.r, blend.r),
    hardLight(base.g, blend.g),
    hardLight(base.b, blend.b),
    base.a + blend.a - base.a * blend.a);
}

vec4 hardLight(vec4 base, vec4 blend, float opacity) {
    return hardLight(base, blend * opacity);
}

float screen(float base, float blend) {
    return base + blend - base * blend;
}

lowp vec4 screen(lowp vec4 base, lowp vec4 blend) {
    return base + blend - base * blend;
}

vec4 screen(vec4 base, vec4 blend, float opacity) {
    return mix(base, screen(base, blend), opacity);
}

void main() {
    vec4 source = texture2D(inputImageTexture, textureCoordinate);
    vec4 blur = texture2D(inputImageTexture2, textureCoordinate);
    lowp vec4 result = source;
    result.rgb = mix(source.rgb, hardLight(source.rgb, blur.rgb), hardlightIntensity);
    result = mix(result, screen(result, blur), screenIntensity);

    result = mix(source, result, mixIntensity);

    gl_FragColor = result;
}