precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

const highp float blurMax = 0.1;
const highp float aberrationMax = 1.2;
const highp int numIters = 20;

uniform highp float intensity;

vec4 getTexture0(vec2 uv){
    return texture2D(inputImageTexture, uv);
}

lowp float weightedColour(lowp float colour, highp int offset, lowp float mid) {
    return colour * 1.0 * (mid - abs(mid - float(offset))) / mid;
}

void main() {
    highp float blur = -blurMax * intensity;
    highp float aberration = 1.0 + (aberrationMax * intensity);

    highp int channelspread = int(float(numIters) / aberration);
    highp float mid = float(channelspread + 1) / 2.0;

    highp ivec3 lobound =
    ivec3(numIters - channelspread, (numIters - channelspread) / 2, 0);
    highp ivec3 hibound =
    ivec3(numIters, (numIters + channelspread) / 2, channelspread);

    highp vec2 centre = vec2(0.5, 0.5);
    highp float len = distance(textureCoordinate, centre);
    highp float angle = atan(textureCoordinate.y - 0.5, textureCoordinate.x - 0.5);
    highp float baseLen = length(vec2(0.2)); // 调整中间清晰的半径
    highp float scaleLen = len - baseLen;
    scaleLen = clamp(scaleLen, 0., 1.);

    highp vec4 colour = vec4(0.0);

    gl_FragColor = getTexture0(textureCoordinate);

    if(len < baseLen) {
        gl_FragColor = getTexture0(textureCoordinate);
        return;
    }

    highp float blurPerIter = blur / float(numIters - 1);
    for (highp int i = 0; i < numIters; i++) {
        highp float scale = 1.0 + (blurPerIter * float(i));
        highp float clen = baseLen + scaleLen * scale;
        highp vec2 cuv = vec2(cos(angle) * clen, sin(angle) * clen) + 0.5;
        lowp vec4 txColour = getTexture0(cuv);
        highp ivec3 kk = ivec3(i + 1) - lobound;

        if (i >= lobound.x && i < hibound.x) {
            colour.x += weightedColour(txColour.x, kk.x, mid);
        }
        if (i >= lobound.y && i < hibound.y) {
            colour.y += weightedColour(txColour.y, kk.y, mid);
        }
        if (i >= lobound.z && i < hibound.z) {
            colour.z += weightedColour(txColour.z, kk.z, mid);
        }
    }

    lowp vec4 source = getTexture0(textureCoordinate);
    colour.rgb = colour.rgb / (0.5*float(channelspread+1));
    colour.a = source.a;

    gl_FragColor = colour;
}