precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform vec4 resolution;
uniform float randomSeed;
uniform lowp int frameIndex;

const float DARK_RANGE = 0.07;

float easeOutQuint(float x) {
    return 1. - pow(1. - x, 5.);
}

float easeInQuint(float x) {
    return x * x * x * x * x;
}

lowp float softLight(lowp float base, lowp float blend) {
    return (blend<0.5)?(2.0*base*blend+base*base*(1.0-2.0*blend)):(sqrt(base)*(2.0*blend-1.0)+2.0*base*(1.0-blend));
}

lowp vec4 softLight(lowp vec4 base, lowp vec4 blend) {
    return vec4(
    softLight(base.r, blend.r),
    softLight(base.g, blend.g),
    softLight(base.b, blend.b),
    base.a + blend.a - base.a * blend.a);
}

vec4 softLight(vec4 base, vec4 blend, float opacity) {
    return mix(base, softLight(base, blend), opacity);
}

float vignetteColor(vec2 uv) {
    float d = length(uv * resolution.xy - 0.5 * resolution.xy) / (sqrt(resolution.x * resolution.x + resolution.y * resolution.y) / 2.);
    float v = 0.95;
    d = (d / v) + (1. - v);
    d = clamp(d, 0., 1.);
    return easeInQuint(d);
}

float rand(vec2 xy, float seed) {
    vec3 p3 = vec3(xy, seed);
    p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.zyx + 31.32);
    return fract((p3.x + p3.y) * p3.z);
}

float rand(vec2 xy) {
    vec3 p3  = fract(vec3(xy.xyx) * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

float rand(float seed) {
    seed = fract(seed * .1031);
    seed *= seed + 33.33;
    seed *= seed + seed;
    return fract(seed);
}

float nextRand(inout float seed) {
    seed = rand(vec2(0.64825 - rand(seed + 3.456), 0.83658 + rand(seed * 0.34)), 1. - seed);
    return seed;
}

vec4 getPin0(vec2 uv) {
    return texture2D(inputImageTexture, uv);
}

void main() {
    float vig = vignetteColor(textureCoordinate);
    //fragColor = softLight(getPin0(uv), vec4(vec3(0.), 1.), vig);
    vec4 result = softLight(getPin0(textureCoordinate), vec4(vec3(0.), 1.), vig);

    gl_FragColor = vec4(result.rgb * (1.0 - DARK_RANGE * rand(randomSeed+float(frameIndex) * 0.5)), result.a);
}
