precision highp float;
precision highp int;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform highp float strenth;
uniform highp float opacity;
uniform highp vec4 rgba;
uniform highp float isGetNotHighlight;

uniform highp float useCustomColor;
//非高光区是否使用自定义颜色
uniform highp float useCustomColorOther;
uniform highp vec4 rgbaOther;

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax - rgbmin;
        hsl.y = mix(delta / (2. - rgbmax - rgbmin), delta / (rgbmax + rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g - rgb.b) / delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b - rgb.r) / delta;
        else hsl.x = 4. + (rgb.r - rgb.g) / delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x + 1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

void main() {
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    vec3 hsl = rgb2hsl(color.rgb);
    float brightness = hsl.z;
    vec4 result = color;
    if (isGetNotHighlight == 1.0) {
        if (brightness <= strenth) {
            if (useCustomColor == 1.0) {
                result = rgba;
            } else {
                result = color;
            }
        } else {
            if (useCustomColorOther == 1.0) {
                result = vec4(rgbaOther.rgb, opacity);
            } else {
                result = vec4(color.rgb, opacity);
            }
        }
    } else {
        if (brightness >= strenth) {
            if (useCustomColor == 1.0) {
                result = rgba;
            } else {
                result = color;
            }
        } else {
            if (useCustomColorOther == 1.0) {
                result = vec4(rgbaOther.rgb, opacity);
            } else {
                result = vec4(color.rgb, opacity);
            }
        }
    }
    gl_FragColor = result;
}