precision highp float;
precision highp int;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float intensity;
uniform highp vec4 rgba;
uniform highp vec4 bgRgba;
uniform int useRgba;
uniform int useBgRgba;

// soft light
float blendSoftLight(float base, float blend) {
    return (blend < 0.5) ? (2.0 * base * blend + base * base * (1.0 - 2.0 * blend)) : (sqrt(base) * (2.0 * blend - 1.0) + 2.0 * base * (1.0 - blend));
}

vec3 blendSoftLight(vec3 base, vec3 blend) {
    return vec3(blendSoftLight(base.r, blend.r), blendSoftLight(base.g, blend.g), blendSoftLight(base.b, blend.b));
}

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax - rgbmin;
        hsl.y = mix(delta / (2. - rgbmax - rgbmin), delta / (rgbmax + rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g - rgb.b) / delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b - rgb.r) / delta;
        else hsl.x = 4. + (rgb.r - rgb.g) / delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x + 1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

void main() {
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    vec3 hsl = rgb2hsl(color.rgb);
    float brightness = hsl.z;
    vec4 base = color;
    if (brightness >= intensity) {
        if (useRgba == 1) {
            base = rgba;
        } else {
            base = color;
        }
    } else {
        if (useBgRgba == 1) {
            base = bgRgba;
        } else {
            base = color;
        }
    }
    gl_FragColor = base;
}