precision highp float;

#define LIMIT 2.0
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float enhance;
uniform float strength;

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax - rgbmin;
        hsl.y = mix(delta / (2. - rgbmax - rgbmin), delta / (rgbmax + rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g - rgb.b) / delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b - rgb.r) / delta;
        else hsl.x = 4. + (rgb.r - rgb.g) / delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x + 1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

vec4 hardLightEnhance(vec4 base, float enhance) {
    return clamp(base * enhance, 0.0, 1.0);
}

void main() {
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    vec3 hsl = rgb2hsl(color.rgb);
    vec4 highlightColor;
    if (hsl.z > strength) {
        float end = (LIMIT - strength) * (1.0-enhance) + strength;
        float enhance = clamp(((hsl.z - strength)/(end-strength) + 1.0), 1.0, 2.0);
        highlightColor = hardLightEnhance(color, enhance);
    } else {
        highlightColor = color;
    }
    gl_FragColor = highlightColor;
}