precision highp float;
precision highp int;

#define PI 3.141592653

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform highp vec2 iResolution;
uniform highp float layer1Blur;
uniform highp float layer2Blur;
uniform highp vec4 layer1RGBA;

vec2 saturate(vec2 pos) {
    return vec2(clamp(pos.x, 0., 1.), clamp(pos.y, 0., 1.));
}

vec2 symmetric(vec2 pos) {
    return 1. - abs(1. - mod(pos, 2.));
}

float rand(vec2 xy) {
    return fract(sin(dot(xy, vec2(12.9898, 78.233))) * 43758.5453123);
}

vec4 gaussianBlurFast1(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 12.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size / resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    float offset = rand(uv);
    if (Size > 0.0) {
        for (float d = 0.0; d < 2. * PI; d += 2. * PI / Directions) {
            for (float i = 1.0 / Quality; i <= 1.0; i += 1.0 / Quality) {
                Color.rgb += texture2D(tex, symmetric(uv + vec2(cos(d + offset), sin(d + offset)) * Radius * i)).rgb;
                count++;
            }
        }
    }
    Color.rgb /= float(count);
    return Color;
}

vec4 gaussianBlurFast2(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 12.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size / resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    float offset = rand(uv);
    if (Size > 0.0) {
        for (float d = 0.0; d < 2. * PI; d += 2. * PI / Directions) {
            for (float i = 1.0 / Quality; i <= 1.0; i += 1.0 / Quality) {
                Color.rgb += texture2D(tex, symmetric(uv + vec2(cos(d + offset), sin(d + offset)) * Radius * i)).rgb;
                count++;
            }
        }
    }
    Color.rgb /= float(count);
    return Color;
}

void main() {
    vec2 uv = textureCoordinate;
    //原图层加颜色模糊
    vec4 base = gaussianBlurFast1(inputImageTexture, uv, iResolution.xy, 25. * layer1Blur);
    //原图层加模糊
    vec4 overlay = gaussianBlurFast2(inputImageTexture, uv, iResolution.xy, 25. * layer2Blur);
    gl_FragColor = base + overlay - base * overlay.a;
}