precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform float sharpness; // 0-3
uniform float radius; // 0.3
uniform float imageWidthFactor;
uniform float imageHeightFactor;

vec3 blurSample(vec2 uv, vec2 xoff, vec2 yoff) {
    vec3 v11 = texture2D(inputImageTexture, uv + xoff).rgb;
    vec3 v12 = texture2D(inputImageTexture, uv + yoff).rgb;
    vec3 v21 = texture2D(inputImageTexture, uv - xoff).rgb;
    vec3 v22 = texture2D(inputImageTexture, uv - yoff).rgb;
    return (v11 + v12 + v21 + v22 + 2.0 * texture2D(inputImageTexture, uv).rgb) * 0.166667;
}

vec3 edgeStrength(vec2 uv) {
    float spread = radius;
    vec2 offset = vec2(1.0) / vec2(imageWidthFactor, imageHeightFactor);
    vec2 up = vec2(0.0, offset.y) * spread;
    vec2 right = vec2(offset.x, 0.0) * spread;

    vec3 v12 = blurSample(uv + up, right, up);

    vec3 v21 = blurSample(uv - right, right, up);
    vec3 v22 = blurSample(uv, right, up);
    vec3 v23 = blurSample(uv + right, right, up);

    vec3 v32 = blurSample(uv - up, right, up);

    vec3 laplacian_of_g =  v12 *  1.0
    + v21 * 1.0 + v22 * -4.0 + v23 * 1.0
    + v32 * 1.0 ;
    laplacian_of_g = laplacian_of_g * (spread / 0.3 * 0.12 + 0.88);
    return laplacian_of_g.xyz;
}

void main() {
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    gl_FragColor = vec4(color.xyz - edgeStrength(textureCoordinate) * sharpness, 1.0);
}
