precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform lowp float intensity;
float blockStep = 64.0;
float row = 8.0;


vec4 sampleAs3DTexture(vec3 textureColor)
{
    //        float step = 64.;
    //        float row = 8.;
    float scanStep = blockStep - 1.;
    float blueColor = textureColor.b * scanStep;
    float rowStep = 1. / row;
    float lutWidth = row * blockStep;
    float xStep = 1. / lutWidth;
    float xHalfStep = 0.5 / lutWidth;

    vec2 quad1;
    quad1.y = floor(floor(blueColor) / row);
    quad1.x = floor(blueColor) - (quad1.y * row);
    vec2 quad2;
    quad2.y = floor(ceil(blueColor) / row);
    quad2.x = ceil(blueColor) - (quad2.y * row);

    vec2 texPos1;
    texPos1.x = (quad1.x * rowStep) + xHalfStep + ((rowStep - xStep) * textureColor.r);
    texPos1.y = (quad1.y * rowStep) + xHalfStep + ((rowStep - xStep) * textureColor.g);

    vec2 texPos2;
    texPos2.x = (quad2.x * rowStep) + xHalfStep + ((rowStep - xStep) * textureColor.r);
    texPos2.y = (quad2.y * rowStep) + xHalfStep + ((rowStep - xStep) * textureColor.g);

    vec4 newColor1 = texture2D(inputImageTexture2, texPos1);
    vec4 newColor2 = texture2D(inputImageTexture2, texPos2);

    vec4 result = mix(newColor1, newColor2, fract(blueColor));
    return newColor1;
}


void main()
{
    vec4 srcColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 filteredColor = sampleAs3DTexture(srcColor.rgb);
    gl_FragColor = mix(srcColor, filteredColor, intensity);
}
