precision highp float;
varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform float noise;
uniform float intensity;
uniform int blendMode;


float sigma = 0.10;

float blendColorBurn(float base, float blend) {
    return (blend==0.0)?blend:max((1.0-((1.0-base)/blend)), 0.0);
}

float blendColorDodge(float base, float blend) {
    return (blend==1.0)?blend:min(base/(1.0-blend), 1.0);
}

float blendVividLight(float base, float blend) {
    return (blend<0.5)?blendColorBurn(base, (2.0*blend)):blendColorDodge(base, (2.0*(blend-0.5)));
}

vec3 blendVividLight(vec3 base, vec3 blend) {
    return vec3(blendVividLight(base.r, blend.r), blendVividLight(base.g, blend.g), blendVividLight(base.b, blend.b));
}

vec3 blendVividLight(vec3 base, vec3 blend, float opacity) {
    return (blendVividLight(base, blend) * opacity + base * (1.0 - opacity));
}


void main() {
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    vec4 color_noise = texture2D(inputImageTexture2, vec2(fract(textureCoordinate2.x + noise), textureCoordinate2.y));
    if (blendMode == 1) {
        vec3 blend_color = blendVividLight(color.rgb, color_noise.rgb, intensity);
        gl_FragColor = vec4(blend_color, 1.0);
    } else {
        vec4 new_color = color + (color_noise - vec4(0.35)) * sigma * intensity;

        vec3 color1 = clamp(new_color.rgb, vec3(0.0), vec3(1.0));
        gl_FragColor = vec4(color1, 1.0);
    }

}