#extension GL_OES_standard_derivatives : enable

precision highp float;

#define PI 6.28318530718
#define TRANSPARENT vec4(0.)
#define WHITE vec4(1.)

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform vec2 iResolution;
uniform float uParams[9];

//void blend(in vec4 src, inout vec4 res, float opacity) {
//    res = vec4(src.rgb*opacity*(1.-step(src.a, 0.))+ (1.-src.a*opacity)*res.rgb, max(res.a, src.a*opacity));
//}
//void blend(in vec4 src, inout vec4 res) {
//    blend(src, res, 1.);
//}

vec4 gaussianBlurFast(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 16.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size/resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    if (Size > 0.0) {
        for (float d=0.0; d< PI ; d+= PI / Directions) {
            for (float i=1.0/Quality; i<=1.0; i+=1.0/Quality) {
                Color.rgb += texture2D(tex, uv+vec2(cos(d), sin(d))*Radius*i).rgb;
                count++;
            }
        }
    }
    Color.rgb /= float(count);
    return Color;
}

void main() {
    vec2 uv = textureCoordinate;
    gl_FragColor = gaussianBlurFast(inputImageTexture, uv, iResolution, 10.*uParams[1]);
}