precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 iChannelResolution;
uniform vec2 uCenter;// 0~1
uniform float uStrength;// 0~1
uniform float uRadius;// 0~1

vec2 platformUV(vec2 uv, vec2 mat) {
    return mat2(uv.x, mat.x, 0., mat.y)*vec2(1., uv.y);
}

vec4 texture2DFlip(sampler2D sampler, vec2 st) {
    return texture2D(sampler, vec2(st.x, st.y));
}

float rand(vec2 xy) {
    return fract(sin(dot(xy, vec2(12.9898, 78.233)))*43758.5453123);
}

vec4 rotateDirectionalBlur(sampler2D samp, vec2 uv, vec2 speed, float strength) {
    vec4 col = vec4(0.0);
    float total = 0.0;
    float offset = rand(uv);
    for (float t = 0.0; t < 8.0; t++) {
        float percent = (t + offset) / 8.0;
        float weight = 4.0 * (percent - percent * percent);
        vec2 off = speed * percent * strength;

        vec2 offFoward = uv + off;
        if(offFoward.x > 1.){
            offFoward.x = 1.;
        }
        if(offFoward.y > 1.){
            offFoward.y > 1.;
        }

        vec2 offBack = uv + off;
        if(offBack.x < 0.){
            offBack.x = 0.;
        }
        if(offBack.y < 0.){
            offBack.y < 0.;
        }

        vec4 color1 = texture2DFlip(inputImageTexture, offFoward)*weight;
        vec4 color2 = texture2DFlip(inputImageTexture, offBack)*weight;
        col += color1+color2;
        total += weight+weight;
    }
    return col / total;
}

void main() {
    vec2 uv = platformUV(textureCoordinate, vec2(0., 1.));
    float ratio = iChannelResolution.x / iChannelResolution.y;
    vec2 pos = uv-uCenter;
    pos.y *= 1./ratio;
    float len = length(pos);
    vec4 color = vec4(0.);

    vec2 relativeCenter = uCenter + normalize(uv-uCenter)*max(uRadius * 0.5-.12, 0.);
    vec2 center = relativeCenter;
    vec2 speed = cross(vec3(uv - center, 0.), vec3(0., 0., -1.)).xy;
    float blur = 0.0305*uStrength;
    color = rotateDirectionalBlur(inputImageTexture, uv, speed, blur);

    vec4 source = texture2DFlip(inputImageTexture, uv);
    color = mix(source, color, smoothstep(-0.01, 0.01, len-uRadius * 0.5));



    gl_FragColor = color;
}