precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 iChannelResolution;
uniform vec2 uCenter;
uniform float uStrength;
uniform float uRadius;

vec2 platformUV(vec2 uv, vec2 mat) {
    return mat2(uv.x, mat.x, 0., mat.y)*vec2(1., uv.y);
}

vec4 texture2DFlip(sampler2D sampler, vec2 st) {
    return texture2D(sampler, vec2(st.x, 1.-st.y));
}

vec3 hsv2rgb(in vec3 c) {
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

vec2 rotateUV(vec2 st, vec2 center, float angle, float ratio) {
    vec2 difference = st - center;
    difference.y *= 1./ratio;
    vec2 dir = normalize(difference);
    float dist = length(difference);
    float deg = radians(angle);
    vec2 rotatedDir = dir*mat2(cos(deg), -sin(deg), sin(deg), cos(deg));
    vec2 rotatedUV = rotatedDir * dist;
    rotatedUV.y *= ratio;
    rotatedUV += center;
    return rotatedUV;
}

vec4 blend(vec4 src, vec4 layer, vec3 alpha) {
    vec4 color = src;
    color.r = mix(src.r, layer.r, alpha.r);
    color.g = mix(src.g, layer.g, alpha.g);
    color.b = mix(src.b, layer.b, alpha.b);
    return color;
}

float easeInCubic(float x) {
    return x * x * x;
}

float easeOutCubic(float x) {
    return 1.0 - pow(1.0 - x, 3.0);
}

vec4 prism(sampler2D samp, vec2 uv, vec2 center) {
    float ratio = iChannelResolution.x/iChannelResolution.y;
    vec2 pos = uv;
    vec4 src = texture2D(samp, pos);
    vec4 color = src;
    int COUNT = 12;
    float colorst = -0.2;
    float colored = -1.;
    for (int i = 0; i < COUNT; i++) {
        vec4 layer = texture2D(samp, rotateUV(uv, center, -10.8*uStrength+float(i)*(12.*uStrength/float(COUNT-1)), ratio));
        float hue = mix(colorst, colored, easeInCubic(float(i)/float(COUNT-1)));
        color = blend(color, layer, hsv2rgb(vec3(hue, 1., 1.)));
    }
    return color;
}

void main() {
    vec2 uv = platformUV(textureCoordinate, vec2(0., 1.));
    float ratio = iChannelResolution.x / iChannelResolution.y;
    vec4 color = vec4(0.);
    vec2 pos = uv-uCenter;
    pos.y *= 1./ratio;
    float len = length(pos);

    vec2 relativeCenter = uCenter + normalize(uv-uCenter)*max(uRadius-.12, 0.);
    color = prism(inputImageTexture, uv, relativeCenter);
    vec4 source = texture2D(inputImageTexture, uv);
    color = mix(source, color, smoothstep(-0.04, 0.04, len-uRadius));

    gl_FragColor = color;
}