precision highp float;

#define PI 3.141592653
#define TRANSPARENT vec4(0.)
#define WHITE vec4(1.)

varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;

uniform vec2 iResolution;

//鱼眼
uniform bool enableFishEye;
uniform bool hasFishEyeOverlay;
uniform float fishEyeIntensity;
uniform float fishEyeRadius;
uniform float fishEyeScale;
uniform float fishEyeRadian;
uniform vec2 fishEyeCenter;
uniform sampler2D fishEyeImage;
uniform float fishEyeOverlayOpacity;
uniform vec2 fishEyeOverlaySize;

//划痕参数
uniform float scaratchOpacity;
uniform bool enableScaratch;
uniform sampler2D scaratchImage;

//漏光参数
uniform float leakOpacity;
uniform bool enableLeak;
uniform sampler2D leakImage;

float screen(float base, float overlay){
    return 1.0 - (1.0 - base) * (1.0 - overlay);
}

// https://www.shadertoy.com/view/4s2GRR
void main() {
    vec2 uv = textureCoordinate;
    vec4 base;
    if(enableFishEye) {
        vec2 m = fishEyeCenter;//fishEyeCenter coords
        vec2 d = uv - m;//vector from fishEyeCenter to current fragment
        d.y *= iResolution.y/iResolution.x*(0.5+fishEyeScale);
        float r = length(d);// distance of pixel from fishEyeCenter
        float sqrt2 = sqrt(2.);

        if (r < fishEyeRadius*sqrt2*.5) {
            r /= fishEyeRadius;
            if (fishEyeIntensity > 0.5) {
                float str = (fishEyeIntensity-0.5)/0.5*0.8;
                float bind = (.5+0.2*fishEyeRadian)*sqrt2;//stick to corners
                float power = PI / (2.0 * bind) * min(0.99, str);//amount of effect
                uv = m + normalize(d) * tan(r * power) * bind / tan(bind * power);//Weird formulas
            } else if (fishEyeIntensity < 0.5) {
                float str = (1.-fishEyeIntensity/0.5)*0.8;
                float bind = (.5+0.2*fishEyeRadian)*sqrt2;
                float power = PI / (2.0 * bind) * min(0.99, str);
                float curv = tan(r * power) * bind / tan(bind * power);
                curv = r*2.-curv;
                uv = m + 0.75*normalize(d) * curv;
            }
        }
    }
    base = texture2D(inputImageTexture, uv);

    if (enableScaratch) {
        vec4 overlay = texture2D(scaratchImage, uv);
        base = vec4(mix(base.rgb, vec3(screen(base.r, overlay.r), screen(base.g, overlay.g), screen(base.b, overlay.b)), overlay.a * scaratchOpacity), base.a);
    }
    if(enableLeak){
        vec4 overlay = texture2D(leakImage, uv);
        base = vec4(mix(base.rgb, vec3(screen(base.r, overlay.r), screen(base.g, overlay.g), screen(base.b, overlay.b)), overlay.a * leakOpacity), base.a);
    }
    if(enableFishEye) {
        if (hasFishEyeOverlay) {
            //vec2 uv2 = textureCoordinate2.xy * iResolution.xy / fishEyeOverlaySize.xy;
            vec4 overlay = texture2D(fishEyeImage, textureCoordinate2) * fishEyeOverlayOpacity;
            gl_FragColor = base + overlay - base * overlay.a;
        } else {
            gl_FragColor = base;
        }
    } else {
        gl_FragColor = base;
    }


}
