precision highp float;

#define PI 3.141592653
#define TRANSPARENT vec4(0.)
#define WHITE vec4(1.)

varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform vec2 iResolution;

uniform float intensity;
uniform float radius;
uniform float scale;
uniform float radian;
uniform vec2 center;

uniform float overlayOpacity;
uniform vec2 overlaySize;
uniform bool hasOverlay;
uniform bool enable;
// https://www.shadertoy.com/view/4s2GRR
void main() {
    vec2 uv = textureCoordinate;
    if (enable){
        vec2 m = center;//center coords
        vec2 d = uv - m;//vector from center to current fragment
        d.y *= iResolution.y/iResolution.x*(0.5+scale);
        float r = length(d);// distance of pixel from center
        float sqrt2 = sqrt(2.);

        if (r < radius*sqrt2*.5) {
            r /= radius;
            if (intensity > 0.5) {
                float str = (intensity-0.5)/0.5*0.8;
                float bind = (.5+0.2*radian)*sqrt2;//stick to corners
                float power = PI / (2.0 * bind) * min(0.99, str);//amount of effect
                uv = m + normalize(d) * tan(r * power) * bind / tan(bind * power);//Weird formulas
            } else if (intensity < 0.5) {
                float str = (1.-intensity/0.5)*0.8;
                float bind = (.5+0.2*radian)*sqrt2;
                float power = PI / (2.0 * bind) * min(0.99, str);
                float curv = tan(r * power) * bind / tan(bind * power);
                curv = r*2.-curv;
                uv = m + 0.75*normalize(d) * curv;
            }
        }

        vec4 color = texture2D(inputImageTexture, uv);

        if (hasOverlay){
            //vec2 uv2 = textureCoordinate2.xy * iResolution.xy / overlaySize.xy;
            vec4 overlay = texture2D(inputImageTexture2, textureCoordinate2) * overlayOpacity;
            gl_FragColor = color + overlay - color * overlay.a;
        } else {
            gl_FragColor = color;
        }
    } else {
        gl_FragColor = texture2D(inputImageTexture, uv);
    }
}
