precision highp float;
precision highp int;

#define PI 3.141592653

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform highp vec2 iResolution;
uniform highp float uBlur;


//uniform sampler2D inputImageTexture2;

//soft_focus相关
uniform float uSoftBlur;
uniform float uSoftBlurBrightness;
uniform float uSoftBlurSaturation;
uniform float uSoftBlurTemperature;
uniform float uSoftBlurTint;

const vec3 WarmFilter = vec3(0.93, 0.54, 0.0);
const mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
const mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);
vec4 saturation(vec4 color, float satAdjust) {
    float luminance = dot(color.rgb, vec3(0.2125, 0.7154, 0.0721));
    color.rgb= mix(vec3(luminance), color.rgb, satAdjust);
    return color;
}

vec4 whitebalance(vec4 color, float temperature, float tint) {
    //adjusting tint
    vec3 yiq = RGBtoYIQ * color.rgb;
    yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
    vec3 rgb = YIQtoRGB * yiq;

    //adjusting temperature
    vec3 processed = vec3(
    (rgb.r < 0.5 ? (2.0 * rgb.r * WarmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - WarmFilter.r))),
    (rgb.g < 0.5 ? (2.0 * rgb.g * WarmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - WarmFilter.g))),
    (rgb.b < 0.5 ? (2.0 * rgb.b * WarmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - WarmFilter.b))));

    color.rgb = mix(rgb, processed, temperature);
    return color;
}

float unionAlpha(float b, float s) {
    return b+s-b*s;
}

float blendNormal(float b, float s) {
    return s;
}
vec3 blendNormal(vec3 b, vec3 s) {
    return s;
}
vec3 blendNormal(vec3 b, vec3 s, float opacity) {
    return mix(b, s, opacity);
}
vec4 blendNormal(vec4 b, vec4 s, float opacity) {
    float a = unionAlpha(b.a, s.a);
    vec3 r = mix(b.rgb, mix(s.rgb, blendNormal(b.rgb, s.rgb, opacity), b.a), s.a/a);
    return vec4(r, a);
}
float blendSoftLight(float b, float s) {
    float d = b>0.25?sqrt(b):(((16.*b-12.)*b+4.)*b);
    return s>0.5?(b+(2.*s-1.)*(d-b)):(b-(1.-2.*s)*b*(1.-b));
}
vec3 blendSoftLight(vec3 b, vec3 s) {
    return vec3(blendSoftLight(b.r, s.r), blendSoftLight(b.g, s.g), blendSoftLight(b.b, s.b));
}
vec3 blendSoftLight(vec3 b, vec3 s, float opacity) {
    return (blendSoftLight(b, s) * opacity + b * (1.0 - opacity));
}
vec4 blendSoftLight(vec4 b, vec4 s, float opacity) {
    float a = unionAlpha(b.a, s.a);
    vec3 r = mix(b.rgb, mix(s.rgb, blendSoftLight(b.rgb, s.rgb, opacity), b.a), s.a/a);
    return vec4(r, a);
}
float blendScreen(float b, float s) {
    return b+s-b*s;
}
vec3 blendScreen(vec3 b, vec3 s) {
    return vec3(blendScreen(b.r, s.r), blendScreen(b.g, s.g), blendScreen(b.b, s.b));
}
vec3 blendScreen(vec3 b, vec3 s, float opacity) {
    return mix(b, blendScreen(b, s), opacity);
}
vec4 blendScreen(vec4 b, vec4 s, float opacity) {
    float a = unionAlpha(b.a, s.a);
    vec3 r = mix(b.rgb, mix(s.rgb, blendScreen(b.rgb, s.rgb, opacity), b.a), s.a/a);
    return vec4(r, a);
}

//fastgauss_blur相关
vec2 saturate(vec2 pos) {
    return vec2(clamp(pos.x, 0., 1.), clamp(pos.y, 0., 1.));
}

vec2 symmetric(vec2 pos) {
    return 1.-abs(1.-mod(pos, 2.));
}

float rand(vec2 xy) {
    return fract(sin(dot(xy, vec2(12.9898, 78.233)))*43758.5453123);
}

vec4 gaussianBlurFast(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 12.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size/resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    float offset = rand(uv);
    if (Size > 0.0) {
        for (float d=0.0; d<2.*PI; d+=2.*PI/Directions) {
            for (float i=1.0/Quality; i<=1.0; i+=1.0/Quality) {
                Color.rgb += texture2D(tex, symmetric(uv+vec2(cos(d+offset), sin(d+offset))*Radius*i)).rgb;
                count++;
            }
        }
    }
    Color.rgb /= float(count);
    return Color;
}

void main() {
    vec2 uv = textureCoordinate;
    vec4 fastGaussColor = gaussianBlurFast(inputImageTexture, uv, iResolution.xy, 25.*uBlur);

    //执行soft_focus
    vec4 source=texture2D(inputImageTexture, uv);
    vec4 blur= fastGaussColor;//texture2D(inputImageTexture2, textureCoordinate);
    source = whitebalance(source, uSoftBlurTemperature, uSoftBlurTint);
    source = saturation(source, uSoftBlurSaturation);
    blur = whitebalance(blur, uSoftBlurTemperature, uSoftBlurTint);
    blur = saturation(blur, uSoftBlurSaturation);
    vec4 result = blendNormal(source, blur, uSoftBlur);
    result = blendScreen(result, blur, uSoftBlurBrightness);

    gl_FragColor = result;
//    gl_FragColor = fastGaussColor;
}