precision highp float;

varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixIntensity;
uniform float blendIntensity;
uniform int enable;

// hard light
float blendHardLight(float base, float blend) {
    //return blend < 0.5 ? (2.0 * base * blend) : (1.0 - 2.0 * (1.0 - base) * (1.0 - blend));
    return (1.0 - 2.0 * (1.0 - base) * (1.0 - blend));
}

vec3 blendHardLight(vec3 base, vec3 blend) {
    return vec3(blendHardLight(base.r, blend.r), blendHardLight(base.g, blend.g), blendHardLight(base.b, blend.b));
}

vec3 blendFunc(vec3 base, vec3 blend, float opacity) {
    return (blendHardLight(base, blend) * opacity + base * (1.0 - opacity));
}

void main() {
    if (enable == 1) {
        vec4 baseColor = texture2D(inputImageTexture, textureCoordinate);
        vec4 overlayColor = texture2D(inputImageTexture2, textureCoordinate2);
        if (overlayColor.a == 1.0) {
            vec4 hardLight = vec4(blendFunc(baseColor.rgb, overlayColor.rgb, blendIntensity), baseColor.a);
            gl_FragColor = mix(baseColor, hardLight, mixIntensity);
        } else {
            gl_FragColor = baseColor;
        }
    } else {
        gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
    }
}
