// FindEdge v1.0
// lujing.zd@bytedance.com 2020_05_13
// SobelEdgeDetection: https://gist.github.com/Hebali/6ebfc66106459aacee6a9fac029d0115

precision highp float;
// uniform
// The original image
uniform sampler2D inputImageTexture;
// varying
// UV
varying highp vec2 kernelUVs[9];
varying highp vec2 textureCoordinate;

vec4 realColor(sampler2D tex, vec2 kernal){
    vec4 n = texture2D(tex, kernal);
    n.rgb /= (n.a + 0.0001);
    return n;
}

void main() {
    // Texture sampling
    lowp vec4 n0 = realColor(inputImageTexture, kernelUVs[0]);
    lowp vec4 n1 = realColor(inputImageTexture, kernelUVs[1]);
    lowp vec4 n2 = realColor(inputImageTexture, kernelUVs[2]);
    lowp vec4 n3 = realColor(inputImageTexture, kernelUVs[3]);
    lowp vec4 n4 = realColor(inputImageTexture, kernelUVs[4]);
    lowp vec4 n5 = realColor(inputImageTexture, kernelUVs[5]);
    lowp vec4 n6 = realColor(inputImageTexture, kernelUVs[6]);
    lowp vec4 n7 = realColor(inputImageTexture, kernelUVs[7]);
    lowp vec4 n8 = realColor(inputImageTexture, kernelUVs[8]);
    // Calculation of the edge
    lowp vec4 sobel_edge_h = n2 + (2.0*n5) + n8 - (n0 + (2.0*n3) + n6);
    lowp vec4 sobel_edge_v = n0 + (2.0*n1) + n2 - (n6 + (2.0*n7) + n8);
    lowp vec4 sobel = sqrt((sobel_edge_h * sobel_edge_h) + (sobel_edge_v * sobel_edge_v));
    sobel = clamp(sobel, 0.0, 1.0);
    lowp float gray = dot(sobel.rgb, vec3(0.299, 0.587, 0.114));
    sobel = sobel * n4 * gray ;
    // Get the portrait mask

    // The return value
    gl_FragColor = vec4(sobel.rgb, 1.0) * texture2D(inputImageTexture, textureCoordinate).a;
}