// OctonEffect PlanA v1.0
// lujing.zd@bytedance.com 2020_05_19
// Scheme A, soft light filter

precision highp float;
// uniform
// Based on the texture
uniform sampler2D inputImageTexture;
// External luminescent texture 发光纹理
uniform sampler2D inputImageTexture2;
// Fuzzy texture 模糊纹理
uniform sampler2D inputImageTexture3;
// The intensity of diffusion扩散强度
uniform float softenIntensity;
// Details of the compensation
uniform float sharpenIntensity;
// The intensity of the special effects 特效强度
uniform float geIntensity;
uniform float intensity;
// varying
// UV
varying highp vec2 textureCoordinate;

// Soft light gray
float softLight(float src, float dst) {
    return src <= 0.5 ?
    dst + (src * 2.0 - 1.0) * (dst - dst * dst) :
    dst + (src * 2.0 - 1.0) * (sqrt(dst) - dst);
}
// Soft light RGB
vec3 softLight(vec3 src, vec3 dst) {
    return vec3(
    softLight(src.x, dst.x),
    softLight(src.y, dst.y),
    softLight(src.z, dst.z)
    );
}
// Soft light RGB and intensity
vec3 softLightByWeight(vec3 src, vec3 dst, float weight) {
    lowp vec3 softlight = softLight(src, dst);
    return mix(dst, softlight, weight);
}
// Screen overlay gray
float screen(float src, float dst) {
    return 1.0 - (1.0 - dst) * (1.0 - src);
}
// Screen overlay RGB
vec3 screen(vec3 src, vec3 dst) {
    return vec3(
    screen(src.x, dst.x),
    screen(src.y, dst.y),
    screen(src.z, dst.z)
    );
}

void main() {
    // Texture sampling
    lowp vec3 base = texture2D(inputImageTexture, textureCoordinate).rgb;
    float alpha = texture2D(inputImageTexture, textureCoordinate).a;
    base /= (alpha + 0.0001);
    lowp vec3 edged = texture2D(inputImageTexture2, textureCoordinate).rgb;
    lowp vec4 var_blured = texture2D(inputImageTexture3, textureCoordinate);
    lowp vec3 blur = var_blured.rgb;
    lowp vec3 bloom = vec3(var_blured.a, var_blured.a, var_blured.a);
    // High contrast retention
    lowp vec3 highPass = vec3(0.5, 0.5, 0.5) + edged - blur;
    // hybrid
    lowp vec3 final = mix(edged, blur, softenIntensity);
    final = softLightByWeight(highPass, final, sharpenIntensity);
    // glow
    final = screen(blur * bloom, final);
    // Specific intensity control
    final = mix(base, final, geIntensity);
    // The return value
//    gl_FragColor = vec4(final, 1.0) * alpha;
//    gl_FragColor = vec4(final,1.0);

    // The return value
    vec4 resCol = vec4(final, 1.0) * alpha;
    resCol.rgb = mix(base, resCol.rgb, intensity);
    gl_FragColor = resCol;

}