precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 iResolution;

uniform int interlaced;
uniform int interlacedLine;
uniform float interlacedLight;
uniform int linenum;

float hash13(vec3 p3)
{
    p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

int randRange(int a, int b) {
    return int(hash13(gl_FragCoord.xyz) * float(b - a) + float(a));
}

vec3 rgb2yuv(float r, float g, float b) {
    return vec3(
    floor(r *  .299000 + g *  .587000 + b *  .114000),
    floor(r * -.168736 + g * -.331264 + b *  .500000 + 128.),
    floor(r *  .500000 + g * -.418688 + b * -.081312 + 128.)
    );
}

vec3 yuv2rgb(float y, float u, float v) {
    return vec3(
    clamp(floor(y + 1.4075 * (v - 128.)), 0., 255.),
    clamp(floor(y - 0.3455 * (u - 128.) - (0.7169 * (v - 128.))), 0., 255.),
    clamp(floor(y + 1.7790 * (u - 128.)), 0., 255.)
    );
}

void main() {
    vec2 uv = textureCoordinate;
    if (linenum > 0) {
        float interlacedLight2 = pow(interlacedLight, 2.);
        float interlacedRatio = float(interlacedLine) / float(iResolution.y);
        float interlacedPeriod = mod(textureCoordinate.y, interlacedRatio) / interlacedRatio;
        float interlacedPixel = abs(interlacedPeriod - .5) * 2.;
        float interlacedLightPeriod = (interlacedPixel - .5) * 2.;
        if(interlaced > 0) {
            float xPixels = floor(uv.x * float(iResolution.x)) + float(interlaced) * interlacedPixel;
            float yPixels = floor(uv.y * float(iResolution.y));
            if (xPixels > float(iResolution.x)) {
                xPixels = mod(xPixels, float(iResolution.x));
                yPixels = yPixels - 1.;
            }
            uv.x = xPixels / float(iResolution.x);
            uv.y = yPixels / float(iResolution.y);
        }
        vec4 color = texture2D(inputImageTexture, uv);
        color.rgb *= 255.;
        color.rgb = rgb2yuv(color.r, color.g, color.b);
        color.r *= (1. + interlacedLight2 * interlacedLightPeriod);
        color.rgb = yuv2rgb(color.r, color.g, color.b) / 255.;
        gl_FragColor = color;
        gl_FragColor = clamp(gl_FragColor, 0.0, gl_FragColor.a);
    } else {
        gl_FragColor = texture2D(inputImageTexture, uv);
    }
}

