precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 iResolution;

uniform float transposeX;
uniform float transposePow;
uniform float transposeNoise;

float hash13(vec3 p3)
{
    p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

int randRange(int a, int b) {
    return int(hash13(gl_FragCoord.xyz) * float(b - a) + float(a));
}

vec3 rgb2yuv(float r, float g, float b) {
    return vec3(
    floor(r *  .299000 + g *  .587000 + b *  .114000),
    floor(r * -.168736 + g * -.331264 + b *  .500000 + 128.),
    floor(r *  .500000 + g * -.418688 + b * -.081312 + 128.)
    );
}

vec3 yuv2rgb(float y, float u, float v) {
    return vec3(
    clamp(floor(y + 1.4075 * (v - 128.)), 0., 255.),
    clamp(floor(y - 0.3455 * (u - 128.) - (0.7169 * (v - 128.))), 0., 255.),
    clamp(floor(y + 1.7790 * (u - 128.)), 0., 255.)
    );
}

void main() {
    vec2 uv = textureCoordinate;
    if(transposeX > 0.0) {
        float xPixels = floor((uv.x + transposeX * pow(1. - uv.y, transposePow) * (1. + transposeNoise * (hash13(gl_FragCoord.xyz) - .5))) * float(iResolution.x));
        float yPixels = floor(uv.y * float(iResolution.y));
        if (xPixels > float(iResolution.x)) {
            xPixels = mod(xPixels, float(iResolution.x));
            yPixels = yPixels - 1.;
        }
        uv.x = xPixels / float(iResolution.x);
        uv.y = yPixels / float(iResolution.y);
    }

    vec4 color = texture2D(inputImageTexture, uv);
    gl_FragColor = color;
    gl_FragColor = clamp(gl_FragColor, 0.0, gl_FragColor.a);
}
