precision highp float;
uniform sampler2D inputImageTexture;
uniform vec2 iResolution;
//uniform vec3 colorRadius;
//uniform vec3 colorSigma;
uniform float maxRadius;
uniform int edgeMode;
uniform int horizontal;
varying highp vec2 textureCoordinate;

vec2 vec4ToVec2(vec4 val)
{
    float a = val.x + val.y / 255.0;
    float b = val.z + val.w / 255.0;
    return vec2(a, b);
}

vec4 vec2ToVec4(vec2 val)
{
    float a = floor(val.x * 255.0) / 255.0;
    float b = fract(val.x * 255.0);
    float c = floor(val.y * 255.0) / 255.0;
    float d = fract(val.y * 255.0);
    return vec4(a, b, c, d);
}

vec2 getS(float a, float b)
{
    vec2 s = vec2(1.0);
    if (edgeMode == 0)
    {
        s.x = step(0.0, a) * step(a, 1.0);
        s.y = step(0.0, b) * step(b, 1.0);
    }
    return s;
}

vec4 neighborColor(float i)
{
    vec2 unit = 1.0 / iResolution.xy;
    if (horizontal == 1) {
        vec2 uv1 = textureCoordinate + unit * vec2(i, 0.);
        vec2 uv2 = textureCoordinate - unit * vec2(i, 0.);
        vec2 s = getS(uv1.x, uv2.x);
        return texture2D(inputImageTexture, uv1) * s.x + texture2D(inputImageTexture, uv2) * s.y;
    } else {
        vec2 uv1 = textureCoordinate + unit * vec2(0., i);
        vec2 uv2 = textureCoordinate - unit * vec2(0., i);
        vec2 s = getS(uv1.y, uv2.y);
        return vec4(vec4ToVec2(texture2D(inputImageTexture, uv1)), 0.0, 0.0) * s.x
        + vec4(vec4ToVec2(texture2D(inputImageTexture, uv2)), 0.0, 0.0) * s.y;
    }
}

float Gaussian(float x, float sigmaSquare)
{
    return exp(-(x * x) / sigmaSquare);
}

    #define ITERATIONS 1024.0
vec4 blurColor(vec4 color)
{
    float widthX = 1.0;
    float widthY = 1.0;
    float widthRed = 1.0;
    float widthGreen = 1.2;
    float widthBlue = 1.4;
    float width = 360.0;
    float glowWidth = 0.1;

    vec3 colorRadius;
    if (horizontal == 1) {
        //水平
        colorRadius = vec3(
        glowWidth * width * widthX * widthRed,
        glowWidth * width * widthX * widthGreen,
        glowWidth * width * widthX * widthBlue
        );
    } else {
        //垂直
        colorRadius = vec3(
        glowWidth * width * widthY * widthRed,
        glowWidth * width * widthY * widthGreen,
        glowWidth * width * widthY * widthBlue
        );
    }
    vec3 colorSigma = vec3(colorRadius.x,colorRadius.y,colorRadius.z) ;
    colorSigma /= 2.4;
    colorSigma = pow(colorSigma, vec3(2.0, 2.0, 2.0)) * 2.0;

    vec4 weight = vec4(1.0);
    vec4 w = vec4(0.0);
    bool loopFlag = true;
    float maxR = max(max(colorRadius.r, colorRadius.g), colorRadius.b);
    float stepI = max(1.0, maxR / maxRadius);
    float i = 1.0;
    for (float j = 1.0; j <= ITERATIONS; j += 1.0)
    {
        if (i > ITERATIONS) break;
        loopFlag = i <= colorRadius.r || i <= colorRadius.g || i <= colorRadius.b;
        if (i > maxRadius*stepI || !loopFlag)
        {
            break;
        }
        w.r = step(i, colorRadius.r) * Gaussian(float(i), colorSigma.r);
        w.g = step(i, colorRadius.g) * Gaussian(float(i), colorSigma.g);
        w.b = step(i, colorRadius.b) * Gaussian(float(i), colorSigma.b);
        color += neighborColor(i) * w;
        weight += w * 2.;

        i += stepI;
    }
    color /= weight;
    return color;
}

void main()
{
    vec4 curColor;
    if (horizontal == 1) {
        curColor = texture2D(inputImageTexture, textureCoordinate);
    } else {
        curColor = vec4(0.0);
        curColor.rg = vec4ToVec2(texture2D(inputImageTexture, textureCoordinate));
    }
    gl_FragColor = vec2ToVec4(blurColor(curColor).rg);
}
