precision highp float;
uniform sampler2D inputImageTexture;
uniform vec4 color;
uniform float intensity;
uniform float glowFromAlpha;
varying highp vec2 textureCoordinate;

#define EPSILON 0.000001


float smoothFuc(float x, float th)
{
    return x <= th ? 0.0 : (x - th) / (1.0 - th);
}

void main()
{
    vec4 inputColor = texture2D(inputImageTexture, textureCoordinate);
    inputColor.rgb /= (inputColor.a+0.0001);

    inputColor.rgb = mix(inputColor.rgb, vec3(1.0), glowFromAlpha);

    // float luminance = dot(inputColor.rgb, vec3(0.299, 0.587, 0.114));
    // float brightness = sigmoid(luminance, 10.0, intensity * 1.1);
    // float brightness = smoothstep(-0.1, 0.1, luminance - (intensity));
    float r = smoothFuc(inputColor.r, intensity + color.r);
    float g = smoothFuc(inputColor.g, intensity + color.g);
    float b = smoothFuc(inputColor.b, intensity + color.b);
    float a = smoothFuc(inputColor.a, intensity);
    float brightness = (r + g + b) / max(inputColor.r + inputColor.g + inputColor.b, EPSILON);
    gl_FragColor = vec4(inputColor.rgb * brightness, 1.0) * a;
}
