precision highp float;
varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
//noiseTexture
uniform sampler2D inputImageTexture2;

uniform vec2 iResolution;
uniform vec2 fullBlendTexSize;
uniform vec2 direction;

uniform float alphaFactor;

#define BLEND_MODE blendColorDodge//
#define BLEND(base, blend, blendFun) blendFun(base, blend)

//color dodge
float blendColorDodge(float base, float blend){
    return (blend==1.)?blend:min(base/(1.-blend), 1.);
}

vec3 blendColorDodge(vec3 base, vec3 blend){
    return vec3(blendColorDodge(base.r, blend.r), blendColorDodge(base.g, blend.g), blendColorDodge(base.b, blend.b));
}

vec3 blendFunc(vec3 base, vec3 blend, float opacity)//opacity
{
    vec3 resultColor= blendColorDodge(base, blend);//BLEND(base, blend, BLEND_MODE);//
    resultColor=mix(base, resultColor, opacity);
    return resultColor;
}

vec4 lm_inverse_premult(vec4 color)
{
    vec4 resultColor=vec4(clamp(color.rgb/color.a, 0.000000, 1.000000), color.a);
    return resultColor;
}

vec2 sucaiAlign(vec2 videoUV, vec2 videoSize, vec2 sucaiSize, vec2 anchorImageCoord, float sucaiScale)
{
    vec2 videoImageCoord=videoUV*videoSize;
    vec2 sucaiUV=(videoImageCoord-anchorImageCoord)/(sucaiSize*sucaiScale)+vec2(.5);
    return sucaiUV;
}

vec4 blendColor(sampler2D sucai, vec4 baseColor, vec2 videoSize, vec2 sucaiSize, vec2 anchorImageCoord, float sucaiScale)
{
    lowp vec4 resultColor=baseColor;
    lowp vec4 sucaiColor=baseColor;
    vec2 sucaiUV=sucaiAlign(textureCoordinate, videoSize, sucaiSize, anchorImageCoord, sucaiScale);
    if (all(lessThan(vec2(0.), sucaiUV))&&all((lessThan(sucaiUV, vec2(1.)))))
    {
        sucaiUV.y = 1.0-sucaiUV.y;//
        sucaiColor=texture2D(sucai, sucaiUV);
        sucaiColor=lm_inverse_premult(sucaiColor);
        resultColor=vec4(blendFunc(baseColor.rgb, sucaiColor.rgb, sucaiColor.a), 1.);
        // resultColor.a = sucaiColor.a + (1.0-sucaiColor.a)*baseColor.a;
        resultColor.a = baseColor.a;
        // resultColor.a = mix(baseColor.a,sucaiColor.a,sucaiColor.a);
        //normal
        // resultColor.a = sucaiColor.a+baseColor.a-baseColor.a*sucaiColor.a;

        //screen
        // resultColor.rgb = baseColor.a*(1.-sucaiColor.rgb)+sucaiColor.rgb;
        // resultColor = baseColor.a*(1.-sucaiColor)+sucaiColor;
    }

    return resultColor;
}

void main(void)
{
    vec2 baseTextureSize=vec2(iResolution.x, iResolution.y);
    vec2 fullBlendAnchor=baseTextureSize*.5;
    float scale=1.;

    //center crop
    float baseAspectRatio=baseTextureSize.y/baseTextureSize.x;
    float blendAspectRatio=fullBlendTexSize.y/fullBlendTexSize.x;
    if (baseAspectRatio>=blendAspectRatio){
        scale=baseTextureSize.y/fullBlendTexSize.y;
    } else {
        scale=baseTextureSize.x/fullBlendTexSize.x;
    }

    lowp vec4 baseColor=texture2D(inputImageTexture, textureCoordinate);

    vec2 screenSize = baseTextureSize;
    vec2 uv_unit = vec2(1.0)/screenSize;
    vec2 uv_offset = uv_unit*direction;
    vec2 uv1 = textureCoordinate+uv_offset;
    lowp vec4 curColor = baseColor;//texture2D(inputImageTexture,textureCoordinate);
    lowp vec4 newColor = texture2D(inputImageTexture, uv1);
    lowp vec4 resultColor = curColor;
    resultColor.r = newColor.r;
    resultColor.g = newColor.g;
    baseColor = resultColor;

    resultColor=blendColor(inputImageTexture2, baseColor, baseTextureSize, fullBlendTexSize, fullBlendAnchor, scale);

    gl_FragColor= resultColor;
}
