precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform float intensitySharp;
uniform vec2 iResolution;

vec4 sharp(vec4 baseColor, float intensity) {
    float surfaceWidth=iResolution.x;
    float surfaceHeight=iResolution.y;

    if (intensity == 0.0) {
        return baseColor;
    }

    vec4 result_color = baseColor;

    float bitmapMaxLength = max(surfaceHeight, surfaceWidth);
    float f = abs(intensity);

    float f2 = (bitmapMaxLength - 1000.0) / 2000.0;
    f2 = max(0.0, min(f2, 1.0));
    f = ((f * 4.0) * (((1.0 - f2) * 0.65) + (f2 * 1.2))) + 1.0;
    float f3 = (1.0 - f) * 0.25;

    vec4 color_left = texture2D(inputImageTexture, textureCoordinate+vec2(-1.0/surfaceWidth, 0.0));
    color_left.rgb = color_left.rgb/(color_left.a+0.001);
    vec4 color_right = texture2D(inputImageTexture, textureCoordinate+vec2(1.0/surfaceWidth, 0.0));
    color_right.rgb = color_right.rgb/(color_right.a+0.001);
    vec4 color_bottom = texture2D(inputImageTexture, textureCoordinate+vec2(0.0, 1.0/surfaceHeight));
    color_bottom.rgb = color_bottom.rgb/(color_bottom.a+0.001);
    vec4 color_top = texture2D(inputImageTexture, textureCoordinate+vec2(0.0, -1.0/surfaceHeight));
    color_top.rgb = color_top.rgb/(color_top.a+0.001);
    result_color.rgb = f*result_color.rgb + f3*color_left.rgb + f3*color_right.rgb + f3*color_top.rgb + f3*color_bottom.rgb;
    result_color = clamp(result_color, 0.0, 1.0);
    return result_color;
}

void main() {
    highp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    if (textureColor.a > 0.0) textureColor.rgb /= textureColor.a;
    float inColorA = textureColor.a;
    // 锐化
    if (abs(intensitySharp) > 0.001) {
        textureColor = sharp(textureColor, intensitySharp);
    }
    gl_FragColor = textureColor;
}