precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 iResolution;
//强度参数,0.001-8，默认8
uniform float intensity;
//大小参数,0.001-25，默认12.5
uniform float size;
//模糊步长，0-8，默认2.4
uniform float blurStep;

vec3 getWeight(vec3 Col1, vec3 Col2, float T)
{
    return max(1. - abs(Col1 - Col2) / (2.5 * T), 0.);
}
vec4 blur(sampler2D inputTexture, vec2 textureCoordinate, float blurRadius, float stepUV, vec2 screenSize, float T)
{
    vec2 unitUV = vec2(stepUV, stepUV) / screenSize;
    vec4 oriCol = texture2D(inputTexture, textureCoordinate);
    vec4 midCol;
    vec3 midweight;
    vec4 sumColor = vec4(0.);
    vec3 sumWeight = vec3(0.0);
    vec2 textureCoordinateA;
    vec2 textureCoordinateB;
    vec4 colorA;
    vec4 colorB;
    vec3 weightA;
    vec3 weightB;
    for (float j = -floor(blurRadius);j <= 7.01; j += 1.0) {
        if (j > blurRadius) {
            break;
        }
        midCol = texture2D(inputTexture, textureCoordinate + vec2(j, 0) * unitUV);
        midweight = getWeight(oriCol.rgb, midCol.rgb, T);
        sumColor.rgb += midCol.rgb * midweight;
        sumWeight += midweight;
        for (float i = 1.;i <= 7.01; i += 1.0)
        {
            if (i > blurRadius) {
                break;
            }
            textureCoordinateA = textureCoordinate + vec2(j, i) * unitUV;
            textureCoordinateB = textureCoordinate + vec2(j, -i) * unitUV;
            colorA = texture2D(inputTexture, textureCoordinateA);
            colorB = texture2D(inputTexture, textureCoordinateB);
            weightA = getWeight(oriCol.rgb, colorA.rgb, T);
            weightB = getWeight(oriCol.rgb, colorB.rgb, T);
            sumColor.rgb += colorA.rgb * weightA;
            sumColor.rgb += colorB.rgb * weightB;
            sumWeight += weightA + weightB;
        }
    }
    vec3 resultCol = clamp(sumColor.rgb / sumWeight, 0., 1.);
    return vec4(resultCol, oriCol.a);
}
void main() {
    vec2 screenSize = vec2(iResolution.xy);
    vec4 col = blur(inputImageTexture, textureCoordinate, intensity, blurStep, screenSize, size / 255.);

    gl_FragColor = vec4(col);
}