precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform float intensityTexture;
uniform vec2 iResolution;
uniform vec2 oilTextureSize;

vec4 blendFunc(vec4 base, vec4 blend){
    vec4 result = vec4(0., 0., 0., 1.);
    result = clamp(mix(2.*blend*base+base*base*(1.-2.*blend), 2.*base*(1.-blend)+sqrt(base)*(2.*blend-1.), step(0.5, blend)), 0., 1.);
    return result;
}

vec2 sucaiAlign(vec2 videoUV,vec2 videoSize,vec2 sucaiSize,vec2 anchorImageCoord,float sucaiScale)
{
    vec2 videoImageCoord = videoUV * videoSize;
    vec2 sucaiUV= (videoImageCoord - anchorImageCoord)/(sucaiSize*sucaiScale) + vec2(0.5);//scaleSize对素材图也就是mask图按比例缩放
    return sucaiUV;
}

vec4 oilTextureBlendColor(sampler2D sucai, vec4 baseColor,vec2 videoSize,vec2 sucaiSize,vec2 anchorImageCoord,float sucaiScale) {
    lowp vec4 resultColor = baseColor;

    vec2 sucaiUV = sucaiAlign(textureCoordinate,videoSize,sucaiSize,anchorImageCoord,sucaiScale);

    lowp vec4 fgColor = baseColor;

    if(sucaiUV.x >= 0.0 && sucaiUV.x <= 1.0 && sucaiUV.y >= 0.0 && sucaiUV.y <= 1.0 ) {
        // sucaiUV.y = 1.0 - sucaiUV.y;
        fgColor = texture2D(sucai,sucaiUV);
    } else {
        return baseColor;
    }

    vec4 result = vec4(0.0);
    fgColor.rgb = fgColor.rgb/fgColor.a;
    result = blendFunc(baseColor, fgColor);
    result = mix(baseColor, result, intensityTexture);
    return result;
}

void main() {
    highp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    if (abs(intensityTexture) > 0.001) {
        vec2 baseTexureSize = vec2(iResolution.x, iResolution.y);
        vec2 fullBlendAnchor = baseTexureSize * vec2(0.5, 0.5);
        float scale = 1.0;
        //外居中对齐
        float baseAspectRatio = baseTexureSize.y/baseTexureSize.x;
        float blendAspectRatio = oilTextureSize.y/oilTextureSize.x;
        if (baseAspectRatio >= blendAspectRatio) {
            scale = baseTexureSize.y / oilTextureSize.y;
        } else {
            scale = baseTexureSize.x / oilTextureSize.x;
        }
        textureColor = oilTextureBlendColor(inputImageTexture2, textureColor, baseTexureSize, oilTextureSize, fullBlendAnchor, scale);
    }
    gl_FragColor = textureColor;
}